-- =============================================================================
-- Copyright © 2016-2017 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Farnoud Farahmand
-- =============================================================================

library ieee;
use ieee.std_logic_1164.all;
use work.ACORN_pkg.all;

entity ACORN_Datapath is
    port(
        clk             : in  std_logic;
	    rst             : in  std_logic;
        key             : in  std_logic_vector(M_SIZE-1 downto 0);
        bdi             : in  std_logic_vector(M_SIZE-1 downto 0);
        bdo             : out std_logic_vector(M_SIZE-1 downto 0);

    --! Control
        en_KeyReg       : in  std_logic;
        L_KeyReg        : in  std_logic;
        rst_KeyReg      : in  std_logic;
        en_StateReg     : in  std_logic;
        rst_StateReg    : in  std_logic;
        ca_set          : in  std_logic;
        cb_set          : in  std_logic;
        sel_LoadKey     : in  std_logic;
        is_final        : in  std_logic;
        sel_decrypt     : in  std_logic;
        tag_match       : out std_logic;
        sel_M           : in  std_logic_vector(2 downto 0)

        );

end entity ACORN_Datapath;

architecture dataflow of ACORN_Datapath is

    signal SReg_in         : std_logic_vector(STATE_SIZE   -1 downto 0);
    signal SReg_out        : std_logic_vector(STATE_SIZE   -1 downto 0);
    signal ca              : std_logic_vector(M_SIZE       -1 downto 0);
    signal cb              : std_logic_vector(M_SIZE       -1 downto 0);
    signal M               : std_logic_vector(M_SIZE       -1 downto 0);
    signal ks              : std_logic_vector(M_SIZE       -1 downto 0);
    signal KeyReg_Sin      : std_logic_vector(M_SIZE       -1 downto 0);
    signal KeyReg_out      : std_logic_vector(PARAM_SIZE   -1 downto 0);
    signal one_sig         : std_logic_vector(M_SIZE       -1 downto 0);
	signal zero_sig        : std_logic_vector(M_SIZE       -1 downto 0);

    signal keyR_din        : std_logic_vector(PARAM_SIZE   -1 downto 0);

begin

--! ===================================================

    ShiftReg_key: entity work.shiftn(behavioral)
            generic map(
                    D_WIDTH => PARAM_SIZE,
                    S_WIDTH => M_SIZE
            )
            port map(
                    d       => keyR_din,
                    enable  => en_KeyReg,
                    load    => L_KeyReg,
                    sin     => KeyReg_Sin,
                    clk     => clk,
                    rst     => rst_KeyReg,
                    q       => KeyReg_out
            );

    State_Reg: entity work.Register_s(behavioral)
                generic map(N => STATE_SIZE)
                port map(
                        d       => SReg_in,
                        enable  => en_StateReg,
                        reset   => rst_StateReg,
                        clock   => clk,
                        q       => SReg_out
                );

    State_Update: entity work.ACORN_StateUpdate(behavior)
        port map(
            s_in    => SReg_out,
            m       => M,
            ca      => ca,
            cb      => cb,
            is_final=> is_final,
            sel_decrypt => sel_decrypt,
            s_out   => SReg_in,
            ks_out  => ks
        );

--! ===================================================

    one_sig     <= (0 => '1', others => '0');
	zero_sig    <= (others => '0');
    ca          <= (others => '1') when ca_set = '1' else (others => '0');
    cb          <= (others => '1') when cb_set = '1' else (others => '0');
    KeyReg_Sin  <= key when (sel_LoadKey = '1') else KeyReg_out(M_SIZE-1 downto 0);

    with sel_M select M <=
        KeyReg_out(M_SIZE-1 downto 0) when "000",
        (KeyReg_out(M_SIZE-1 downto 0) xor one_sig) when "001",
        bdi      when "010",
		one_sig  when "011",
		zero_sig when others;

    tag_match <= '1' when (bdi = ks) else '0';
    bdo <= ks;


end dataflow;
