-- =============================================================================
-- Copyright © 2016-2017 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Farnoud Farahmand
-- =============================================================================

library ieee;
use ieee.std_logic_1164.all;
use work.ACORN_pkg.all;

entity ACORN_StateUpdate is

    port(
        s_in    : in std_logic_vector(STATE_SIZE-1 downto 0);
        m       : in std_logic_vector(M_SIZE-1 downto 0);
        ca      : in std_logic_vector(M_SIZE-1 downto 0);
        cb      : in std_logic_vector(M_SIZE-1 downto 0);
        is_final: in std_logic;
        sel_decrypt : in std_logic;

        s_out   : out std_logic_vector(STATE_SIZE-1 downto 0);
        ks_out  : out std_logic_vector(M_SIZE-1 downto 0)
    );

end ACORN_StateUpdate;

architecture behavior of ACORN_StateUpdate is

    signal s, s1            : std_logic_vector(M_SIZE+STATE_SIZE-1 downto 0);
    signal ks, f, new_blk, msg, zero_sig  : std_logic_vector(M_SIZE-1     downto 0);

begin

    zero_sig    <= (others => '0');
    s   <= zero_sig & s_in(STATE_SIZE-1 downto 0);
    msg <= (m xor ks) when sel_decrypt='1' else m;
    
    s1(M_SIZE+STATE_SIZE-1 downto 289+M_SIZE)  <=  s(M_SIZE+STATE_SIZE-1 downto 289+M_SIZE);
    s1(288                 downto 230+M_SIZE)  <=  s(288                 downto 230+M_SIZE);
    s1(229                 downto 193+M_SIZE)  <=  s(229                 downto 193+M_SIZE);
    s1(192                 downto 154+M_SIZE)  <=  s(192                 downto 154+M_SIZE);
    s1(153                 downto 107+M_SIZE)  <=  s(153                 downto 107+M_SIZE);
    s1(106                 downto 61 +M_SIZE)  <=  s(106                 downto 61 +M_SIZE);
    s1(60                  downto 0)           <=  s(60                  downto 0);

    Gen_State: for i in 0 to M_SIZE-1 generate

        s1(289+i) <= s(289+i) xor s(235+i) xor s(230+i);
        s1(230+i) <= s(230+i) xor s(196+i) xor s(193+i);
        s1(193+i) <= s(193+i) xor s(160+i) xor s(154+i);
        s1(154+i) <= s(154+i) xor s(111+i) xor s(107+i);
        s1(107+i) <= s(107+i) xor s(66 +i) xor s(61 +i);
        s1(61 +i) <= s(61 +i) xor s(23 +i) xor s(0  +i);

        ks(i) <= KSG128(s(12 +i), s1(154+i), s(235+i), s1(61 +i),
                        s1(193+i), s1(230+i), s(111+i), s(66 +i));

        f(i)  <= FBK128(s(0  +i), s1(107+i), s(244+i), s(23 +i),
                        s(160+i), s(196+i), ca(i), cb(i), ks(i));

        new_blk(i) <= f(i) xor msg(i) xor s1(STATE_SIZE + i);

    end generate Gen_State;

    s_out  <= new_blk & s1(STATE_SIZE-1 downto M_SIZE);
    ks_out <= ks when (is_final = '1') else (ks xor m);

end behavior;
